
/********************************************
rexp1.c

libmawk changes (C) 2009-2010, Tibor 'Igor2' Palinkas;
based on mawk code coming with the below copyright:

copyright 1991, Michael D. Brennan

This is a source file for mawk, an implementation of
the AWK programming language.

Mawk is distributed without warranty under the terms of
the GNU General Public License, version 2, 1991.
********************************************/

/*  re machine	operations  */

#include  "rexp.h"

/* initialize a two state machine */
static void new_TWO(int type, MACHINE *mp)
{
	mp->start = (STATE *) mawk_RE_malloc(2 * STATESZ);
	mp->stop = mp->start + 1;
	mp->start->type = type;
	mp->stop->type = M_ACCEPT;
}

/*  build a machine that recognizes any	 */
MACHINE mawk_RE_any(void)
{
	MACHINE x;

	new_TWO(M_ANY, &x);
	return x;
}

/*  build a machine that recognizes the start of string	 */
MACHINE mawk_RE_start(void)
{
	MACHINE x;

	new_TWO(M_START, &x);
	return x;
}

MACHINE mawk_RE_end(void)
{
	MACHINE x;

	new_TWO(M_END, &x);
	return x;
}

/*  build a machine that recognizes a class  */
MACHINE mawk_RE_class(BV *bvp)
{
	MACHINE x;

	new_TWO(M_CLASS, &x);
	x.start->data.bvp = bvp;
	return x;
}

MACHINE mawk_RE_u(void)
{
	MACHINE x;

	new_TWO(M_U, &x);
	return x;
}

MACHINE mawk_RE_str(char *str, unsigned len)
{
	MACHINE x;

	new_TWO(M_STR, &x);
	x.start->len = len;
	x.start->data.str = str;
	return x;
}


/*  replace m and n by a machine that recognizes  mn   */
void mawk_RE_cat(MACHINE *mp, MACHINE *np)
{
	unsigned sz1, sz2, sz;

	sz1 = mp->stop - mp->start;
	sz2 = np->stop - np->start + 1;
	sz = sz1 + sz2;

	mp->start = (STATE *) mawk_RE_realloc(mp->start, sz * STATESZ);
	mp->stop = mp->start + (sz - 1);
	memcpy(mp->start + sz1, np->start, sz2 * STATESZ);
	free(np->start);
}

 /*  replace m by a machine that recognizes m|n  */

void mawk_RE_or(MACHINE *mp, MACHINE *np)
{
	register STATE *p;
	unsigned szm, szn;

	szm = mp->stop - mp->start + 1;
	szn = np->stop - np->start + 1;

	p = (STATE *) mawk_RE_malloc((szm + szn + 1) * STATESZ);
	memcpy(p + 1, mp->start, szm * STATESZ);
	free(mp->start);
	mp->start = p;
	(mp->stop = p + szm + szn)->type = M_ACCEPT;
	p->type = M_2JA;
	p->data.jump = szm + 1;
	memcpy(p + szm + 1, np->start, szn * STATESZ);
	free(np->start);
	(p += szm)->type = M_1J;
	p->data.jump = szn;
}

/*  UNARY  OPERATIONS	  */

/*  replace m by m*   */
void mawk_RE_close(MACHINE *mp)
{
	register STATE *p;
	unsigned sz;

	sz = mp->stop - mp->start + 1;
	p = (STATE *) mawk_RE_malloc((sz + 2) * STATESZ);
	memcpy(p + 1, mp->start, sz * STATESZ);
	free(mp->start);
	mp->start = p;
	mp->stop = p + (sz + 1);
	p->type = M_2JA;
	p->data.jump = sz + 1;
	(p += sz)->type = M_2JB;
	p->data.jump = -(sz - 1);
	(p + 1)->type = M_ACCEPT;
}

/*  replace m  by  m+  (positive closure)   */
void mawk_RE_poscl(MACHINE *mp)
{
	register STATE *p;
	unsigned sz;

	sz = mp->stop - mp->start + 1;
	mp->start = p = (STATE *) mawk_RE_realloc(mp->start, (sz + 1) * STATESZ);
	mp->stop = p + sz;
	p += --sz;
	p->type = M_2JB;
	p->data.jump = -sz;
	(p + 1)->type = M_ACCEPT;
}

/* replace  m  by  m? (zero or one)  */
void mawk_RE_01(MACHINE *mp)
{
	unsigned sz;
	register STATE *p;

	sz = mp->stop - mp->start + 1;
	p = (STATE *) mawk_RE_malloc((sz + 1) * STATESZ);
	memcpy(p + 1, mp->start, sz * STATESZ);
	free(mp->start);
	mp->start = p;
	mp->stop = p + sz;
	p->type = M_2JB;
	p->data.jump = sz;
}

/*===================================
MEMORY	ALLOCATION
 *==============================*/


PTR mawk_RE_malloc(unsigned sz)
{
	register PTR p;

	if (!(p = malloc(sz)))
		mawk_RE_error_trap(MEMORY_FAILURE);
	return p;
}

PTR mawk_RE_realloc(register PTR p, unsigned sz)
{
	if (!(p = realloc(p, sz)))
		mawk_RE_error_trap(MEMORY_FAILURE);
	return p;
}
