#include <libmawk.h>

CELL *blobb(mawk_state_t *context, CELL * sp, int a_args)
{
	int n;
	char buff[64];

	/* do something - print BLOBB and all arguments */
	printf("BLOBB! ");
	for(n = a_args-1; n >= 0; n--)
		printf("%d='%s' ", n, libmawk_print_cell((sp-n), buff, sizeof(buff)));
	printf("\n");

	/* restore the stack (remove all arguments) */
	sp = sp - a_args;

	/* set a return value (find out where the return value is on the stack,
	   using libmawk_stackret()) */
	libmawk_set_cell(context, libmawk_stackret(sp), 'f', (double)1234);

	/* return the new stack pointer - should be the one that it was before
	   arguments had been pushed on the stack */
	return sp;
}

int main(int argc, char **argv)
{
	mawk_state_t *m;
	CELL *ret;
	char buff[64];

	m = libmawk_initialize(argc, argv);
	if (m != NULL) {
		/* test function call */
		ret = libmawk_call_function(m, "func", "dfs", (int)42, (double)1.234, (char *)"test string.");
		printf("Return value of func is '%s'\n", libmawk_print_cell(ret, buff, sizeof(buff)));
		libmawk_cell_destroy(m, ret);

		/* this is the same function call with a different syntax */
		{
			int i = 42;
			double d = 1.234;
			char *s = "test string.";
			void *args[] = {&i, &d, s};
			ret = libmawk_call_functionp(m, "func", "dfs", args);
			printf("Return value of func is '%s'\n", libmawk_print_cell(ret, buff, sizeof(buff)));
			libmawk_cell_destroy(m, ret);
		}


		/* register a C function (resolved runtime) */
		if (libmawk_register_function(m, "blobb", blobb) != 0) {
			fprintf(stderr, "ERROR: Unable to register function blobb\n");
		}

		/* run some data */
		libmawk_append_input(m, "This is a\nmultiline test input\nput in the artificial input buffer.\n");
		libmawk_run_main(m);

		/* print var */
		ret = libmawk_get_var(m, "var");
		printf("Variable var = '%s'\n", libmawk_print_cell(ret, buff, sizeof(buff)));

		/* run some more data */
		libmawk_append_input(m, "Second\n");
		libmawk_append_input(m, "run.\n");
		libmawk_run_main(m);

		/* run end */
		libmawk_uninitialize(m);
	}
	else {
		printf("Init failed.\n");
	}
	printf("END\n");
	return 0 ;
}

